#========================================================================================
# package      DE_EPAGES::Customer::t::unit::CustomerResource
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Customer::API::Resource::CustomerResource.
#========================================================================================
package DE_EPAGES::Customer::t::unit::CustomerResource;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Customer::API::CustomerService;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_get();

    done_testing();
}



sub test_get {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MockObjects = $self->_createMocks();

    my $Resource = $MockObjects->get('SUT');
    my $Customer = $MockObjects->get('Customer');

    my $hCustomerData = $Resource->get($Customer);
    my $hExpectedCustomerData = $self->_createCustomerData();

    is_deeply($hCustomerData, $hExpectedCustomerData, "$SubTestName - test if expected customer data was returned.");
}



sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['CustomerService'], sub {
        my ($CustomerService) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Customer::API::Resource::CustomerResource', []);
        $Mockify->addMockWithReturnValue('_createCustomerService', $CustomerService);
        return $Mockify->getMockObject();
    });

    $Mockery->register('CustomerService', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Customer::API::CustomerService', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('getCustomerData', $self->_createCustomerData(), ['object']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Customer', [], sub {
        return DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Customer::API::Object::Customer', {});
    });

    return $Mockery->assemble();
}



sub _createCustomerData {
    return {
        'id' => 'test customer'
    };
}

__PACKAGE__->runScript();